<?php

namespace Drupal\decoupled_cookie_auth\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Routing\TrustedRedirectResponse;
use Drupal\Core\Url;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpKernel\Event\ResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Class RedirectResponseSubscriber handles the kernel.response event.
 */
class RedirectResponseSubscriber implements EventSubscriberInterface {

  /**
   * The immutable config.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * Constructs a new RedirectResponseSubscriber object.
   */
  public function __construct(ConfigFactoryInterface $config_factory) {
    $this->config = $config_factory->get('decoupled_cookie_auth.configuration');
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[KernelEvents::RESPONSE] = ['handleResponse'];

    return $events;
  }

  /**
   * This method is called when the kernel.response is dispatched.
   *
   * Handle the redirect after the user clicks the password reset link in an
   * email. The destination is the frontend password reset form, including the
   * pass-reset-token query parameter.
   *
   * @param \Symfony\Component\HttpKernel\Event\ResponseEvent $event
   *   The dispatched event.
   */
  public function handleResponse(ResponseEvent $event) {
    $response = $event->getResponse();
    if ($response instanceof RedirectResponse) {
      $query = [];
      parse_str(parse_url($response->getTargetUrl(), PHP_URL_QUERY), $query);
      if (!empty($query['pass-reset-token'])) {
        $front_end_url = $this->config->get('frontend_url');
        $root_relative_path = $this->config->get('frontend_reset_password_path');
        $url = Url::fromUri($front_end_url . $root_relative_path)
          ->setOption('query', ['pass-reset-token' => $query['pass-reset-token']]);
        $event->setResponse(new TrustedRedirectResponse($url->toString()));
      }
      elseif (strpos($response->getTargetUrl(), '/user/password') !== FALSE) {
        // The password reset link was invalid. Redirect the user to the
        // frontend path to request a new password reset link.
        $front_end_url = $this->config->get('frontend_url');
        $request_password_reset_path = $this->config->get('frontend_request_password_reset_path');
        $url = Url::fromUri($front_end_url . $request_password_reset_path)
          ->setOption('query', ['link_invalid' => 1]);
        $event->setResponse(new TrustedRedirectResponse($url->toString()));
      }
    }
  }

}
